#!/usr/bin/env perl
use strict;
use warnings;
use 5.10.0;

use lib '/usr/share/longview/lib';

use JSON::PP;
use Sys::Hostname;
use LWP::UserAgent;
use Compress::Zlib;
use Linode::Longview::DataGetter qw(get);
use Linode::Longview::Util qw(constantPush flattenData slurp_file daemonize_self check_already_running $SLEEP_TIME);

die "LongView must be run as root in order to collect data\n" unless ($< == 0);
my $pid = check_already_running();
die "The LongView agent is already running as PID: $pid\n" if $pid;

my $ua = LWP::UserAgent->new(
	timeout => 10,
	agent   => "Linode Longview 1.0"
);
my $url = 'https://longview1.linode.com/cgi.pl';

my $stats = {
	apikey  => scalar(slurp_file('/etc/longview.key')),
	version => '1.0',
	payload => [],
};
die "No API key found\n" unless ($stats->{apikey});

my ($debug, $quit, $needSysInfo) = (0,0,0);
daemonize_self() unless $debug;
$SIG{TERM} = $SIG{INT} = $SIG{HUP} = $SIG{QUIT} = sub { $quit = 1 };

my $data={};
my ($counter, $sleep) = (0, $SLEEP_TIME);
while (!$quit) {
	$data->{timestamp} = time;
	get($_,$data) foreach (qw(Processes Uptime Ports Load Memory CPU Network Disk));
	if (!($counter % 1440) || $needSysInfo) {
		$needSysInfo = 0;
		get('SysInfo',$data);
		get('Packages', $data);
	}

	constantPush($stats->{payload},$data);
	$data = {};

	my $req = $ua->post(
		$url,
		Content_Type => 'form-data',
		Content => [
			'data' => [
				undef, 
				'json.gz', 
				'Content-Type' => 'application/json',
				'Content-Encoding' => 'gzip',
				'Content' => Compress::Zlib::memGzip(encode_json($stats))
			]   
		]   
	);  

	$sleep = $SLEEP_TIME;
	if ($req->is_success){
		@{$stats->{payload}} = ();
		my $rep = decode_json($req->decoded_content);
		$sleep = $rep->{sleep} if defined $rep->{sleep};
		$needSysInfo = 1 if defined $rep->{needSysInfo};
	}
	if ($debug) {
		print $req->status_line . "\n";
		print $req->decoded_content . "\n" ;
		exit 0;
	}

	$counter++;
	sleep $sleep;
}
